# -*- coding: utf-8 -*-

# Copyright 2018 Spanish National Research Council (CSIC)
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import aiohttp.test_utils
from aiohttp import web
import fixtures
from oslo_config import cfg
from oslo_config import fixture as config_fixture
from oslo_log import log as logging
import testtools

CONF = cfg.CONF
logging.register_options(CONF)


_TRUE_VALUES = ("True", "true", "1", "yes")


class TestCase(testtools.TestCase, aiohttp.test_utils.AioHTTPTestCase):

    """Base unit test class."""

    async def get_application(self):
        app = web.Application(debug=True)
        app.middlewares.append(web.normalize_path_middleware())

        return app

    def setUp(self):
        """Run before each test method to initialize test environment."""

        super(TestCase, self).setUp()

        test_timeout = os.environ.get("TESTR_TEST_TIMEOUT", 0)
        try:
            test_timeout = int(test_timeout)
        except ValueError:
            # If timeout value is invalid do not set a timeout.
            test_timeout = 0
        if test_timeout > 0:
            self.useFixture(fixtures.Timeout(test_timeout, gentle=True))

        self.useFixture(config_fixture.Config(CONF))

        if os.environ.get("TESTR_STDOUT_CAPTURE") in _TRUE_VALUES:
            stdout = self.useFixture(fixtures.StringStream("stdout")).stream
            self.useFixture(fixtures.MonkeyPatch("sys.stdout", stdout))
        if os.environ.get("TESTR_STDERR_CAPTURE") in _TRUE_VALUES:
            stderr = self.useFixture(fixtures.StringStream("stderr")).stream
            self.useFixture(fixtures.MonkeyPatch("sys.stderr", stderr))

    async def asyncSetUp(self):
        """Run before each test method to initialize test environment."""

        await super(TestCase, self).asyncSetUp()
        self.assertEqual(self.app.debug, True)

    def flags(self, **kw):
        """Override flag variables for a test."""
        group = kw.pop("group", None)
        for k, v in kw.items():
            CONF.set_override(k, v, group)
